/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *									     *
 * Library:	libsubdiv2.5D						     *
 *									     *
 * Header:	subdiv2.5D_priv.h					     *
 *									     *
 * Description:	private interface common to all modules of library	     *
 *									     *
 * Date:	20000313						     *
 *									     *
 * Author:	Francisco MORN BURGOS   (http://www.gti.ssr.upm.es./~fmb/)  *
 *		Grupo Tratamiento Imgenes, Univ. Politcnica Madrid, Spain  *
 *									     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#ifndef _INC_SUBDIV25D_PRIV_
#define _INC_SUBDIV25D_PRIV_



#include "subdiv2.5D.h"
#ifdef DIME
#  include "Plugin.h"
#endif



/* * * * *     GLOBAL CONSTANTS, MACROES, TYPES & VARIABLES     * * * * */

#include <stdlib.h>
#define NEW(n, item)	(item *) calloc(n, sizeof(item))
#define DEL(itemptr)	{free(itemptr); itemptr = NULL;}

/* "M_PI" and "M_PI_2" are not strictly ANSI-standard */
#ifndef M_PI
#  define M_PI		3.14159265358979323846F
#  define M_PI_2	M_PI/2.0F
#endif
#define DEG		* M_PI/180.0F

#define MOGOLLON	8E+8
#define EPSILON		8E-8
#define ABS(x)		(x>0? x : -x)
#define NEGLIGIBLE(x)	(ABS(x) < EPSILON)
#define SGN(x)		(NEGLIGIBLE(x)? 0 : x>0? +1 : -1)

#define BEEP		fprintf(stderr, "\a")

#define IDX_VTX_IN_T(pt, t) \
  (t->vtx[0]==pt? 0 : t->vtx[1]==pt? 1 : t->vtx[2]==pt? 2 : -1)

#define INC_VEC(v1, v2)	{v1.x += v2.x;	v1.y += v2.y;	v1.z += v2.z;}
#define DEC_VEC(v1, v2)	{v1.x -= v2.x;	v1.y -= v2.y;	v1.z -= v2.z;}
#define MUL_VEC(v1, k)	{v1.x *= k;	v1.y *= k;	v1.z *= k;}
#define DIV_VEC(v1, k)	{v1.x /= k;	v1.y /= k;	v1.z /= k;}

/* leave the definition of "BIT_SHUFFLING" below to have details coded	*/
/* following the bit pattern described by "codedelta'"; comment it out	*/
/* to have them coded as in "codedelta"					*/
/* codedelta =	 N(sign)  N(NBPDN-2)  N(NBPDN-3) ...  N(0)	        */
/*		T1(sign) T1(NBPDT-2) T1(NBPDT-3) ... T1(0)      	*/
/*		T2(sign) T2(NBPDT-2) T2(NBPDT-3) ... T2(0).	        */
/* codedelta' = N(sign) T1(sign) T2(sign) N(NBPDN-2) N(NBPDN-3)	...	*/
/*		N(NBPDT-1) N(NBPDT-2) T1(NBPDT-2) T2(NBPDT-2)		*/
/*		N(NBPDT-3) T1(NBPDT-3) T2(NBPDT-3) ... N(0) T1(0) T2(0)	*/
#define BIT_SHUFFLING

/* "sd{P,M}1" will be initialised in "subdiv2.5D_txitxa.c" so that, for	*/
/* i = {0,1,2}, sdP1[i] = (i+1) % 3 and sdM1[i] = (i-1) % 3, and will	*/
/* never be touched again -- they are like (hopefully fast) macroes...	*/
extern int sdP1[3], sdM1[3];

/* "sdNullVector" will also be initialised in "subdiv2.5D_txitxa.c" to	*/
/* be worth no more than {0.0F, 0.0F, 0.0F}				*/
extern sdVector sdNullVector;



/* * * * *     FUNCTION PROTOTYPES     * * * * */

/* from "subdiv2.5D_gfx.c" */
extern void sdgDisplayMainMesh(void);

/* from "subdiv2.5D_io.c" */
extern void sdiReadDetailsFileHeader(sdMeshPtr m);
extern void sdiReadDetail(sdMeshPtr m, CODEWORD *codedelta, sdVectorPtr delta);
extern sdBoolean sdiReadNextBitFromDetailsFile(sdMeshPtr m);
extern void sdiReadDetailsFile(sdMeshPtr m, sdBoolean animate);
extern void sdiWriteDetailsFileHeader(sdMeshPtr m);
extern void sdiWriteDetail(sdMeshPtr m, CODEWORD *codedelta, sdVectorPtr delta);
extern void sdiWriteNextBitToDetailsFile(sdMeshPtr m, sdBoolean b);
extern void sdiWriteDetailsFile(sdMeshPtr m);
extern void sdiPrintPoint(sdMeshPtr m, sdPointPtr p,
			  sdOutputFormat format, sdBoolean normal);
extern void sdiPrintTriangle(sdMeshPtr m, sdTrianglePtr t,
			     sdOutputFormat format, sdBoolean normal);
extern void sdiPrintAllDetailTrees(sdMeshPtr m);
extern int sdiError(char *msg, int rc);

/* from "subdiv2.5D_lists.c" */
extern sdDetailTree sdlNewLeafDetailTree(char *id);
extern void sdlCalculateMaxDetailsInAllDetailTrees(sdMeshPtr m);
extern void sdlDeleteLeafDetailTree(sdDetailTree *dt);
extern void sdlAddDetailTree2DTreeList(sdDetailTree dt, sdBoolean A,
				       sdDTreeList *dtl_1st,
				       sdDTreeList *dtl_last);
extern void sdlDeleteDTreeListFromDTreeList(sdDTreeList *dtl,
					    sdDTreeList *dtl_1st,
					    sdDTreeList *dtl_last);
extern void sdlDestroyDTreeList(sdDTreeList *dtl);
extern sdPointPtr sdlAddPoint2VertexOT(sdMeshPtr m, sdOctree *ot,
				       sdVectorPtr v, sdBoolean *found);
extern void sdlDeletePointFromVertexOT(sdMeshPtr m, sdOctree *ot,
				       sdPointPtr *pt);
extern void sdlPrintAllPointsInVertexOT(sdMeshPtr m, sdOutputFormat format,
					sdBoolean normal);
extern void sdlTransferAllVerticesFromVertexOT2VertexList(sdMeshPtr m);
extern sdPointPtr sdlAddPoint2VertexList(sdMeshPtr m, sdVectorPtr pt);
extern void sdlDeletePointFromVertexList(sdMeshPtr m, sdPointPtr *pt);
extern sdPointPtr sdlFindVertexByID(sdMeshPtr m, int id);
extern int sdlVertexDepth(sdMeshPtr m, sdPointPtr pt);
extern void sdlComputeAllVertexNormals(sdMeshPtr m);
extern sdTrianglePtr sdlNewLeafTriangle(sdMeshPtr m, int depth);
extern void sdlDeleteLeafTriangle(sdMeshPtr m, sdTrianglePtr *t);
extern sdTrianglePtr sdlFindLeafTriangleByID(sdMeshPtr m, int id);
extern void sdlAddTriangle2TriangleList(sdTrianglePtr t, sdTriangleLst *tl);
extern sdBoolean sdlIsTriangleInTriangleList(sdTrianglePtr t, sdTriangleLst tl);
extern void sdlDestroyTriangleList(sdTriangleLst *tl);
extern void sdlBuildAdjacencies(sdMeshPtr m, sdBoolean keepET);
extern int sdlFindPathBetweenPtsInsidePlane(sdMeshPtr m, sdPointPtr pa,
					    sdPointPtr pb, sdVectorPtr normal,
					    void *from, sdVectorLst *path);

/* from "subdiv2.5D_math.c" */
extern sdVectorPtr sdmMidpoint(sdVectorPtr pa, sdVectorPtr pb);
extern sdVectorPtr sdmNormalise(sdVectorPtr pt);
extern float sdmDotProd(sdVectorPtr a, sdVectorPtr b);
extern sdVectorPtr sdmVecDiff(sdVectorPtr a, sdVectorPtr b);
extern sdVectorPtr sdmVecProd(sdVectorPtr a, sdVectorPtr b);
extern float sdmVecNorm(sdVectorPtr pt);
extern void sdmFillTriangleNormalAndK(sdTrianglePtr t, sdBoolean KisArea);
extern void sdmFillVertexNormal(sdPointPtr vtx, int n, sdPointPtr *nbr);
extern sdBoolean sdmCoplanarNeighbours(sdTrianglePtr t, float epsilon);
extern sdVectorPtr sdmButterflyMidpoint(sdTrianglePtr t, int k,
					sdBoolean initCall);
extern sdVectorPtr sdmMatrixVecProd(sdVectorPtr m, sdVectorPtr v);
extern void sdmMatrixTrans(sdVectorPtr m, sdVectorPtr mt);
extern sdVectorPtr sdmCalculateDetail(sdMeshPtr m, sdPointPtr pt,
				      sdVectorPtr dir);
extern void sdmBuildFrameForTriangleEdge(sdTrianglePtr t, int i, sdVectorPtr v);
extern void sdmEncodeDetail(sdMeshPtr m, int nbits, sdVectorPtr delta,
			    CODEWORD *codedelta);
extern void sdmDecodeDetail(sdMeshPtr m, int nbits, CODEWORD codedelta,
			    sdVectorPtr delta);

#endif /* _INC_SUBDIV25D_PRIV_ */

